/*
 * Copyright (c) 2003 Marcus Glocker
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <sys/types.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <ctype.h>
#include <stdarg.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>

#include "../libmy/file.h"

#include "proto.h"
#include "extern.h"

int
file_rmline(const char *src, const char *file)
{
	FILE	*sfile, *tfile;
	int	i = 1, l;
	char	b[1024], tmp[1024];

	l = fparse(tmp, src, file, sizeof(tmp));

	if ((sfile = fopen(file, "r")) == NULL)
		return -1;
	if ((tfile = fopen("bot.tmp", "w")) == NULL)
		return -1;

	while (fread(b, 1, 1, sfile)) {
		if (b[0] == '\n')
			i++;
		if (i != l)
			fwrite(b, 1, 1, tfile);
	}	

	fclose(sfile);
	fclose(tfile);

	rename("bot.tmp", file);

	return 0;
}

int
file_chline(const char *src, const char *ch, const char *file)
{
	FILE	*sfile, *tfile;
	int	i = 1, j = 0, l;
	char	b[1024], tmp[1024];

	l = fparse(tmp, src, file, sizeof(tmp));

	if ((sfile = fopen(file, "r")) == NULL)
		return -1;
	if ((tfile = fopen("bot.tmp", "w")) == NULL)
		return -1;

	while (fread(b, 1, 1, sfile)) {
		if (b[0] == '\n')
			i++;
		if (i != l)
			fwrite(b, 1, 1, tfile);
		if (i == l && j == 0) {
			fwrite(b, 1, 1, tfile);
			fprintf(tfile, "%s", ch);
			j = 1;
		}
	}	

	fclose(sfile);
	fclose(tfile);

	rename("bot.tmp", file);

	return 0;
}

int
file_write(const char *src, const char *file)
{
	FILE	*sfile;

	if ((sfile = fopen(file, "a")) == NULL)
		return -1;

	fprintf(sfile, "%s", src);

	fclose(sfile);

	return 0;
}

int
file_tabs(char *dst, const char *src, const int row, const int dsize)
{
	int	i, len, tabs;
	float	calc1, calc2;

	len = strlen(src);

	calc1 = row - len;
	calc2 = calc1 / 8;
	tabs = ceilf(calc2);

	for (i = 0; i < tabs && i != dsize-1; i++)
		dst[i] = '\t';

	/* terminate string */
	dst[i] = '\0';

	return 0;
}

int
file_lower(char *dst, const char *src, const int dsize)
{
	int     i;

	for(i = 0; src[i] != '\0' && i != dsize-1; i++)
		dst[i] = tolower(src[i]);

	/* terminate string */
	dst[i] = '\0';

	return 0;
}
