/*
 * Copyright (c) 2003 Marcus Glocker
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *    - Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above
 *      copyright notice, this list of conditions and the following
 *      disclaimer in the documentation and/or other materials provided
 *      with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <ctype.h>

#include "../libmy/file.h"

#include "config.h"
#include "proto.h"
#include "extern.h"

int		ppid;
struct entry 	modtab[MODTAB];

int
sys_daemon()
{
	pid_t	pid;

	if ((pid = fork()) < 0)
		exit(1);
	if (pid > 0)
		exit(0);
	setsid();
	if ((pid = fork()) < 0)
                exit(1);
        if (pid > 0)
                exit(0);
	ppid = getpid();
	flogd(config.filelog, "*** bot started (pid %d)\n", ppid);
	if (chdir(config.botroot) != 0) {
		perror("chdir");
		return -1;
	}
	umask(022);	

	setproctitle(config.botnick);
	
	return 0;
}

int
sys_modtab_load(void)
{
	FILE	*file;
	int	i;
	char	mtfile[128], line[1024], in[6], ch[32], mo[32], op[32];

	snprintf(mtfile, sizeof(mtfile), "%s/modtab", config.moddir);

	if ((file = fopen(mtfile, "r")) == NULL)
		return -1;

	for (i = 0; i < MODTAB; i++) {
		modtab[i].timer = 0;
		modtab[i].interval = 0;
		memset(modtab[i].channel, 0, sizeof(modtab[i].channel));
		memset(modtab[i].module, 0, sizeof(modtab[i].module));
		memset(modtab[i].option, 0, sizeof(modtab[i].option));
	}

	i = 0;
	while (fgets(line, sizeof(line), file) != NULL && i < MODTAB) {
		sscanf(line, "%5s %31s %31s %31s", in, ch, mo, op);
		if ((modtab[i].interval = atoi(in)) == 0)
			continue;
		strlcpy(modtab[i].channel, ch, sizeof(modtab[i].channel));
		strlcpy(modtab[i].module, mo, sizeof(modtab[i].module));
		if (strcmp(op, "-") != 0) {
			modtab[i].option[0] = ' ';
			strlcat(modtab[i].option, op, sizeof(modtab[i].option));
		}
		i++;
	}

	fclose(file);

	return 0;
}

int
sys_modtab_check(const int sfd)
{
	int		i;
	char		cmd[1024];
	time_t		tnow;
	struct tm	*t;

	time(&tnow);
	t = localtime(&tnow);

	if (t->tm_sec != 0)
		return 0;

	for (i = 0; modtab[i].interval != 0 && i < MODTAB; i++) {
		modtab[i].timer++;
		if (modtab[i].interval == modtab[i].timer) {
			flogd(config.filelog,
			    "*** ModTab requests module [%s]\n",
			    modtab[i].module);
			snprintf(cmd, sizeof(cmd),
			    ":%s!module PRIVMSG %s :!%s%s\r\n",
			    config.botnick, modtab[i].channel,
			    modtab[i].module, modtab[i].option);
			irc_privpub(cmd, sfd);
			modtab[i].timer = 0;
		}
	}

	return 0;
}
